package javax.swing.text;

import java.lang.reflect.Constructor;
import java.text.ParseException;
import sun.swing.SwingUtilities2;
import sun.reflect.misc.ReflectUtil;
import java.io.Serializable;
import javax.swing.JFormattedTextField;

public class DefaultFormatter extends JFormattedTextField.AbstractFormatter implements Cloneable, Serializable
{
    private boolean allowsInvalid;
    private boolean overwriteMode;
    private boolean commitOnEdit;
    private Class<?> valueClass;
    private NavigationFilter navigationFilter;
    private DocumentFilter documentFilter;
    transient DefaultFormatter.ReplaceHolder replaceHolder;
    
    public DefaultFormatter() {
        this.overwriteMode = true;
        this.allowsInvalid = true;
    }
    
    @Override
    public void install(JFormattedTextField ftf) {
        super.install(ftf);
        this.positionCursorAtInitialLocation();
    }
    
    public void setCommitsOnValidEdit(boolean commitOnEdit) {
        this.commitOnEdit = commitOnEdit;
    }
    
    public boolean getCommitsOnValidEdit() {
        return this.commitOnEdit;
    }
    
    public void setOverwriteMode(boolean overwriteMode) {
        this.overwriteMode = overwriteMode;
    }
    
    public boolean getOverwriteMode() {
        return this.overwriteMode;
    }
    
    public void setAllowsInvalid(boolean allowsInvalid) {
        this.allowsInvalid = allowsInvalid;
    }
    
    public boolean getAllowsInvalid() {
        return this.allowsInvalid;
    }
    
    public void setValueClass(Class<?> valueClass) {
        this.valueClass = valueClass;
    }
    
    public Class<?> getValueClass() {
        return this.valueClass;
    }
    
    @Override
    public Object stringToValue(String s) throws ParseException {
        Class<Object> clazz = this.getValueClass();
        JFormattedTextField formattedTextField = this.getFormattedTextField();
        if (clazz == null && formattedTextField != null) {
            Object value = formattedTextField.getValue();
            if (value != null) {
                clazz = value.getClass();
            }
        }
        if (clazz != null) {
            Constructor<Object> constructor;
            try {
                ReflectUtil.checkPackageAccess(clazz);
                SwingUtilities2.checkAccess(clazz.getModifiers());
                constructor = clazz.getConstructor(new Class[] { String.class });
            }
            catch (NoSuchMethodException ex) {
                constructor = null;
            }
            if (constructor != null) {
                try {
                    SwingUtilities2.checkAccess(constructor.getModifiers());
                    return constructor.newInstance(new Object[] { s });
                }
                catch (Throwable t) {
                    throw new ParseException("Error creating instance", 0);
                }
            }
        }
        return s;
    }
    
    @Override
    public String valueToString(Object o) throws ParseException {
        if (o == null) {
            return "";
        }
        return o.toString();
    }
    
    @Override
    protected DocumentFilter getDocumentFilter() {
        if (this.documentFilter == null) {
            this.documentFilter = new DefaultFormatter.DefaultDocumentFilter();
        }
        return this.documentFilter;
    }
    
    @Override
    protected NavigationFilter getNavigationFilter() {
        if (this.navigationFilter == null) {
            this.navigationFilter = new DefaultFormatter.DefaultNavigationFilter();
        }
        return this.navigationFilter;
    }
    
    public Object clone() throws CloneNotSupportedException {
        DefaultFormatter defaultFormatter = (DefaultFormatter)super.clone();
        defaultFormatter.navigationFilter = null;
        defaultFormatter.documentFilter = null;
        defaultFormatter.replaceHolder = null;
        return defaultFormatter;
    }
    
    void positionCursorAtInitialLocation() {
        JFormattedTextField formattedTextField = this.getFormattedTextField();
        if (formattedTextField != null) {
            formattedTextField.setCaretPosition(this.getInitialVisualPosition());
        }
    }
    
    int getInitialVisualPosition() {
        return this.getNextNavigatableChar(0, 1);
    }
    
    boolean isNavigatable(int n) {
        return true;
    }
    
    boolean isLegalInsertText(String s) {
        return true;
    }
    
    private int getNextNavigatableChar(int n, int n2) {
        int length = this.getFormattedTextField().getDocument().getLength();
        while (n >= 0 && n < length) {
            if (this.isNavigatable(n)) {
                return n;
            }
            n += n2;
        }
        return n;
    }
    
    String getReplaceString(int endIndex, int n, String str) {
        String text = this.getFormattedTextField().getText();
        String s = text.substring(0, endIndex);
        if (str != null) {
            s += str;
        }
        if (endIndex + n < text.length()) {
            s += text.substring(endIndex + n);
            return s;
        }
        return s;
    }
    
    boolean isValidEdit(DefaultFormatter.ReplaceHolder replaceHolder) {
        if (!this.getAllowsInvalid()) {
            String replaceString = this.getReplaceString(replaceHolder.offset, replaceHolder.length, replaceHolder.text);
            try {
                replaceHolder.value = this.stringToValue(replaceString);
                return true;
            }
            catch (ParseException ex) {
                return false;
            }
        }
        else {
            return true;
        }
    }
    
    void commitEdit() throws ParseException {
        JFormattedTextField formattedTextField = this.getFormattedTextField();
        if (formattedTextField != null) {
            formattedTextField.commitEdit();
        }
    }
    
    void updateValue() {
        this.updateValue(null);
    }
    
    void updateValue(Object stringToValue) {
        try {
            if (stringToValue == null) {
                stringToValue = this.stringToValue(this.getFormattedTextField().getText());
            }
            if (this.getCommitsOnValidEdit()) {
                this.commitEdit();
            }
            this.setEditValid(true);
        }
        catch (ParseException ex) {
            this.setEditValid(false);
        }
    }
    
    int getNextCursorPosition(int n, int n2) {
        int n3 = this.getNextNavigatableChar(n, n2);
        int length = this.getFormattedTextField().getDocument().getLength();
        if (!this.getAllowsInvalid()) {
            if (n2 == -1 && n == n3) {
                n3 = this.getNextNavigatableChar(n3, 1);
                if (n3 >= length) {
                    n3 = n;
                }
            }
            else if (n2 == 1 && n3 >= length) {
                n3 = this.getNextNavigatableChar(length - 1, -1);
                if (n3 < length) {
                    ++n3;
                }
            }
        }
        return n3;
    }
    
    void repositionCursor(int n, int n2) {
        this.getFormattedTextField().getCaret().setDot(this.getNextCursorPosition(n, n2));
    }
    
    int getNextVisualPositionFrom(JTextComponent textComponent, int n, Position.Bias bias, int n2, Position.Bias[] array) throws BadLocationException {
        int n3 = textComponent.getUI().getNextVisualPositionFrom(textComponent, n, bias, n2, array);
        if (n3 == -1) {
            return -1;
        }
        if (!this.getAllowsInvalid() && (n2 == 3 || n2 == 7)) {
            int n4;
            for (n4 = -1; !this.isNavigatable(n3) && n3 != n4; n4 = n3, n3 = textComponent.getUI().getNextVisualPositionFrom(textComponent, n3, bias, n2, array)) {}
            int length = this.getFormattedTextField().getDocument().getLength();
            if (n4 == n3 || n3 == length) {
                if (n3 == 0) {
                    array[0] = Position.Bias.Forward;
                    n3 = this.getInitialVisualPosition();
                }
                if (n3 >= length && length > 0) {
                    array[0] = Position.Bias.Forward;
                    n3 = this.getNextNavigatableChar(length - 1, -1) + 1;
                }
            }
        }
        return n3;
    }
    
    boolean canReplace(DefaultFormatter.ReplaceHolder replaceHolder) {
        return this.isValidEdit(replaceHolder);
    }
    
    void replace(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) throws BadLocationException {
        this.replace(this.getReplaceHolder(filterBypass, n, n2, s, set));
    }
    
    boolean replace(DefaultFormatter.ReplaceHolder replaceHolder) throws BadLocationException {
        boolean n = true;
        int n2 = 1;
        if (replaceHolder.length > 0 && (replaceHolder.text == null || replaceHolder.text.length() == 0) && (this.getFormattedTextField().getSelectionStart() != replaceHolder.offset || replaceHolder.length > 1)) {
            n2 = -1;
        }
        if (this.getOverwriteMode() && replaceHolder.text != null && this.getFormattedTextField().getSelectedText() == null) {
            replaceHolder.length = Math.min(Math.max(replaceHolder.length, replaceHolder.text.length()), replaceHolder.fb.getDocument().getLength() - replaceHolder.offset);
        }
        if ((replaceHolder.text != null && !this.isLegalInsertText(replaceHolder.text)) || !this.canReplace(replaceHolder) || (replaceHolder.length == 0 && (replaceHolder.text == null || replaceHolder.text.length() == 0))) {
            n = false;
        }
        if (!(n == 0)) {
            int n3 = replaceHolder.cursorPosition;
            replaceHolder.fb.replace(replaceHolder.offset, replaceHolder.length, replaceHolder.text, replaceHolder.attrs);
            if (n3 == -1) {
                n3 = replaceHolder.offset;
                if (n2 == 1 && replaceHolder.text != null) {
                    n3 = replaceHolder.offset + replaceHolder.text.length();
                }
            }
            this.updateValue(replaceHolder.value);
            this.repositionCursor(n3, n2);
            return true;
        }
        else {
            this.invalidEdit();
            return false;
        }
    }
    
    void setDot(NavigationFilter.FilterBypass filterBypass, int n, Position.Bias bias) {
        filterBypass.setDot(n, bias);
    }
    
    void moveDot(NavigationFilter.FilterBypass filterBypass, int n, Position.Bias bias) {
        filterBypass.moveDot(n, bias);
    }
    
    DefaultFormatter.ReplaceHolder getReplaceHolder(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) {
        if (this.replaceHolder == null) {
            this.replaceHolder = new DefaultFormatter.ReplaceHolder();
        }
        this.replaceHolder.reset(filterBypass, n, n2, s, set);
        return this.replaceHolder;
    }
    
    static JFormattedTextField access$200(DefaultFormatter defaultFormatter) {
        return defaultFormatter.getFormattedTextField();
    }
    
    static JFormattedTextField access$300(DefaultFormatter defaultFormatter) {
        return defaultFormatter.getFormattedTextField();
    }
    
    static JFormattedTextField access$400(DefaultFormatter defaultFormatter) {
        return defaultFormatter.getFormattedTextField();
    }
    
    static JFormattedTextField access$500(DefaultFormatter defaultFormatter) {
        return defaultFormatter.getFormattedTextField();
    }
    
    static JFormattedTextField access$600(DefaultFormatter defaultFormatter) {
        return defaultFormatter.getFormattedTextField();
    }
    
    static class ReplaceHolder
    {
        DocumentFilter.FilterBypass fb;
        int offset;
        int length;
        String text;
        AttributeSet attrs;
        Object value;
        int cursorPosition;
        
        void reset(DocumentFilter.FilterBypass fb, int offset, int length, String text, AttributeSet attrs) {
            this.fb = fb;
            this.offset = offset;
            this.length = length;
            this.text = text;
            this.attrs = attrs;
            this.value = null;
            this.cursorPosition = -1;
        }
    }
    
    private class DefaultNavigationFilter extends NavigationFilter implements Serializable
    {
        @Override
        public void setDot(NavigationFilter.FilterBypass filterBypass, int n, Position.Bias bias) {
            if (DefaultFormatter.access$200(DefaultFormatter.this).composedTextExists()) {
                filterBypass.setDot(n, bias);
            }
            else {
                DefaultFormatter.this.setDot(filterBypass, n, bias);
            }
        }
        
        @Override
        public void moveDot(NavigationFilter.FilterBypass filterBypass, int n, Position.Bias bias) {
            if (DefaultFormatter.access$300(DefaultFormatter.this).composedTextExists()) {
                filterBypass.moveDot(n, bias);
            }
            else {
                DefaultFormatter.this.moveDot(filterBypass, n, bias);
            }
        }
        
        @Override
        public int getNextVisualPositionFrom(JTextComponent textComponent, int n, Position.Bias bias, int n2, Position.Bias[] array) throws BadLocationException {
            if (textComponent.composedTextExists()) {
                return textComponent.getUI().getNextVisualPositionFrom(textComponent, n, bias, n2, array);
            }
            return DefaultFormatter.this.getNextVisualPositionFrom(textComponent, n, bias, n2, array);
        }
    }
    
    private class DefaultDocumentFilter extends DocumentFilter implements Serializable
    {
        @Override
        public void remove(DocumentFilter.FilterBypass filterBypass, int n, int n2) throws BadLocationException {
            if (DefaultFormatter.access$400(DefaultFormatter.this).composedTextExists()) {
                filterBypass.remove(n, n2);
            }
            else {
                DefaultFormatter.this.replace(filterBypass, n, n2, null, null);
            }
        }
        
        @Override
        public void insertString(DocumentFilter.FilterBypass filterBypass, int n, String s, AttributeSet set) throws BadLocationException {
            if (DefaultFormatter.access$500(DefaultFormatter.this).composedTextExists() || Utilities.isComposedTextAttributeDefined(set)) {
                filterBypass.insertString(n, s, set);
            }
            else {
                DefaultFormatter.this.replace(filterBypass, n, 0, s, set);
            }
        }
        
        @Override
        public void replace(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) throws BadLocationException {
            if (DefaultFormatter.access$600(DefaultFormatter.this).composedTextExists() || Utilities.isComposedTextAttributeDefined(set)) {
                filterBypass.replace(n, n2, s, set);
            }
            else {
                DefaultFormatter.this.replace(filterBypass, n, n2, s, set);
            }
        }
    }
}
